
class Hash

  # Converts all keys in the Hash to be Symbol values, returning a new Hash.
  # With a from_class parameter, limits conversion to only a certain class of keys.
  # It defaults to String --use nil to convert any key class.
  #
  #   foo = { :name=>'Gavin', 'wife'=>:Lisa }
  #   foo.keys_to_sym    #=>  { :name=>"Gavin", :wife=>:Lisa }
  #   foo.inspect        #=>  { "name" =>"Gavin", "wife"=>:Lisa }
  #
  #--
  # Adapted from Gavin Sinclair's graceous work #symbolify_keys.
  #++
  def keys_to_sym( from_class=String )
    raise ArgumentError, "Parameter must be a class" unless from_class.kind_of?(Class) if from_class
    self.dup.keys_to_sym!( from_class )
  end

  alias_method( :symbolize_keys, :keys_to_sym )

  #--
  # # Rails has these aliases too, but they are not very good for
  # # gerenal use, IMHO. But perhaps someone can convince me otherwise.
  # alias_method( :to_options,  :symbolize_keys )
  #++


  # Synonym for Hash#keys_to_symbol, but modifies the receiver in place (and returns it).
  # With a from_class parameter, limits conversion to only a certain class of keys.
  # It defaults to String --use nil to convert any key class.
  #
  #   foo = { 'name'=>'Gavin', 'wife'=>:Lisa }
  #   foo.keys_to_sym!    #=>  { :name=>"Gavin", :wife=>:Lisa }
  #   foo.inspect         #=>  { :name=>"Gavin", :wife=>:Lisa }
  #
  #--
  # Adapted from Gavin Sinclair's graceous work #symbolify_keys!
  #++
  def keys_to_sym!( from_class=String )
    raise ArgumentError, "Parameter must be a class" unless from_class.kind_of?(Class) if from_class
    if from_class
      self.each_key{ |k|
        self[k.to_sym]=self.delete(k) if k.respond_to?(:to_sym) and k.class == from_class
      }
    else
      self.each_key{ |k|
        self[k.to_sym]=self.delete(k) if k.respond_to?(:to_sym)
      }
    end
    self
  end

  alias_method( :symbolize_keys!, :keys_to_sym! )

  #--
  # # Rails has these aliases too, but they are not very good for
  # # gerenal use, IMHO. But perhaps someone can convince me otherwise.
  # alias_method( :to_options!, :symbolize_keys! )
  #++

end


#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#
=begin test

  require 'test/unit'

  class TCHash < Test::Unit::TestCase

    def test_keys_to_sym
      foo = { 'a'=>1, 'b'=>2 }
      assert_equal( { :a=>1, :b=>2 }, foo.keys_to_sym )
      assert_equal( { "a" =>1, "b"=>2 }, foo )
    end

    def test_keys_to_sym!
      foo = { 'a'=>1, 'b'=>2 }
      assert_equal( { :a=>1, :b=>2 }, foo.keys_to_sym! )
      assert_equal( { :a=>1, :b=>2 }, foo  )
    end

  end

=end
