#--
# Inflect
#
# Copyright (c) 2005 Thomas Sawyer
#
# Ruby License
#
# This module is free software. You may use, modify, and/or redistribute this
# software under the same terms as Ruby.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.
#
#================================================================================
# Revision History
#================================================================================
#  2005-10-25  trans
#================================================================================
#++

#:title: Inflect
#
# Collection of Facets that apply to inflecting a String or Symbol.
#
#   "dog".plural          #=> "dogs"
#   "dog_tail".camelcase  #=> "DogTail"
#   "1".ordinal           #=> "1st"
#
#   :going.upcase         #=> :GOING
#   :Arrow.capitlized?    #=> true
#
# = Author(s)
#
# * Thomas Sawyer
# * (see Facets for corollary)
#

require 'facet/string/singular'
require 'facet/string/plural'

require 'facet/string/camelcase'
require 'facet/string/camelize'

require 'facet/string/underscore'
require 'facet/string/humanize'

require 'facet/string/basename'
require 'facet/string/demodulize'
require 'facet/string/modulize'
require 'facet/string/methodize'
require 'facet/string/pathize'

require 'facet/string/upcase'
require 'facet/string/downcase'
require 'facet/string/capitalized'

require 'facet/string/ordinal'

# don't exist, should they?
#require 'facet/symbol/singular'
#require 'facet/symbol/plural'

require 'facet/symbol/camelcase'
require 'facet/symbol/camelize'

require 'facet/symbol/underscore'

require 'facet/symbol/upcase'
require 'facet/symbol/downcase'
require 'facet/symbol/capitalize'
require 'facet/symbol/capitalized'

