
require 'facet/core/kernel/instance'

class OpenStruct

  # Intercepts some methods to apply to the inner table
  # rather then be defined as part of the open members.
  # As with Kernel#instance all other methods are passed
  # on to the Kernel binding.
  #
  #   o = OpenStruct.new
  #   o.a = 1
  #   o.b = 2
  #   o.instance.each { |k, v| puts "#{k} #{v}" }
  #
  # produces
  # 
  #   a 1
  #   b 2   
  #
  # Methods supported:
  #   store, fetch, merge, update, each, key?
  #

  def instance
    Functor.new { |op, *args|
      case op
      when  :[], :[]=, :store, :fetch, :merge, :update,
            :each, :key?
        @table.send(op,*args)
      else
        super(op,*args)  # send(op,*args)
      end
    }
  end

end
