
class String

  SINGULAR_RULES = [
    [/(matr)ices$/i, '\1ix'],
    [/(vert)ices$/i, '\1ex'],
    [/^(ox)en/i, '\1'],
    [/(alias)es$/i, '\1'],
    [/([octop|vir])i$/i, '\1us'],
    [/(cris|ax|test)es$/i, '\1is'],
    [/(shoe)s$/i, '\1'],
    [/(o)es$/i, '\1'],
    [/(bus)es$/i, '\1'],
    [/([m|l])ice$/i, '\1ouse'],
    [/(x|ch|ss|sh)es$/i, '\1'],
    [/(m)ovies$/i, '\1\2ovie'],
    [/(s)eries$/i, '\1\2eries'],
    [/([^aeiouy]|qu)ies$/i, '\1y'],
    [/([lr])ves$/i, '\1f'],
    [/(tive)s$/i, '\1'],
    [/(hive)s$/i, '\1'],
    [/([^f])ves$/i, '\1fe'],
    [/(^analy)ses$/i, '\1sis'],
    [/((a)naly|(b)a|(d)iagno|(p)arenthe|(p)rogno|(s)ynop|(t)he)ses$/i, '\1\2sis'],
    [/([ti])a$/i, '\1um'],
    [/(p)eople$/i, '\1\2erson'],
    [/(m)en$/i, '\1an'],
    [/(s)tatus$/i, '\1\2tatus'],
    [/(c)hildren$/i, '\1\2hild'],
    [/(n)ews$/i, '\1\2ews'],
    [/s$/i, '']
  ]

  PLURAL_RULES = [
    [/^(ox)$/i, '\1\2en'],                 # ox
    [/([m|l])ouse$/i, '\1ice'],            # mouse, louse
    [/(matr|vert)ix|ex$/i, '\1ices'],      # matrix, vertex, index
    [/(x|ch|ss|sh)$/i, '\1es'],            # search, switch, fix, box, process, address
    [/([^aeiouy]|qu)ies$/i, '\1y'],
    [/([^aeiouy]|qu)y$/i, '\1ies'],        # query, ability, agency
    [/(hive)$/i, '\1s'],                   # archive, hive
    [/(?:([^f])fe|([lr])f)$/i, '\1\2ves'], # half, safe, wife
    [/sis$/i, 'ses'],                      # basis, diagnosis
    [/([ti])um$/i, '\1a'],                 # datum, medium
    [/(p)erson$/i, '\1eople'],             # person, salesperson
    [/(m)an$/i, '\1en'],                   # man, woman, spokesman
    [/(c)hild$/i, '\1hildren'],            # child
    [/(buffal|tomat|potat)o$/i, '\1\2oes'],      # buffalo, tomato
    [/(bu)s$/i, '\1\2ses'],                # bus
    [/(alias)/i, '\1es'],                  # alias
    [/(octop|vir)us$/i, '\1i'],            # octopus, virus - virus has no defined plural, but viri is better than viruses/viruss
    [/(ax|cri|test)is$/i, '\1es'],         # axis, crisis  
    [/s$/i, 's'],                          # no change (compatibility)
    [/$/, 's']
  ]


  # Convert an English word from plurel to singular.
  #
  #   "boys".singular      #=> boy
  #   "tomatoes".singular  #=> tomato
  #
  def singular
    result = self.to_s.dup
    String::SINGULAR_RULES.each do |(rule, replacement)|
      break if result.gsub!(rule, replacement)
    end
    return result
  end

  # Convert an English word from singular to plurel.
  #
  #   "boy".plural     #=> boys
  #   "tomato".plural  #=> tomatoes
  #
  def plural
    result = self.to_s.dup
    String::PLURAL_RULES.each do |(rule, replacement)|
      break if result.gsub!(rule, replacement)
    end
    return result
  end

  # Deprecated alias for #singular.
  alias_method( :singularize, :singular )

  # Deprecated alias for #plural.
  alias_method( :pluralize, :plural )

end


#  _____         _
# |_   _|__  ___| |_
#   | |/ _ \/ __| __|
#   | |  __/\__ \ |_
#   |_|\___||___/\__|
#
=begin test

  require 'test/unit'

  class TCString < Test::Unit::TestCase

    def test_singular
      assert_equal( "friend", "friends".singular )
      assert_equal( "man", "men".singular )    
    end

    def test_plural
      assert_equal( "friends", "friend".plural )
      assert_equal( "men", "man".plural )
    end

  end

=end
