# = one.rb
#
# == Copyright (c) 2005 Thomas Sawyer
#
#   Ruby License
#
#   This module is free software. You may use, modify, and/or redistribute this
#   software under the same terms as Ruby.
#
#   This program is distributed in the hope that it will be useful, but WITHOUT
#   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#   FOR A PARTICULAR PURPOSE.
#
# == Special Thanks
#
# Big shout-out to all the authors of fabulous one-liners!
#
# == Author(s)
#
# * Thomas Sawyer

# Author::    Thomas Sawyer
# Copyright:: Copyright (c) 2005 Thomas Sawyer
# License::   Ruby License

require 'yaml'

# = One
#
# One liners helper library provides one letter
# alias for a number of classes, modules and
# methods, allowing one to create very tight one-liners.
# This this bit of a "gag" library, but it can also
# be used for code obsfuscation and includes examples
# of some neat one liners.

# Alias Kernel
K=Kernel

# Alias Object
O=Object

# Alias Class
C=Class

module Kernel
  alias pr print
  alias ps puts
  alias s sleep
  alias l loop
end

module Enumerable
  alias m map
end

class Array
  alias e each
  alias r reverse
end

class String
  alias e each
  alias r reverse
end

class Integer
  alias t times
end

# This module contains a number of historic one-liners.
#
# NOTE Most of these one-liners do not use the one.rb 
# shortcut aliases.

module RubyOneLiners
  extend self

  ONELINERS = YAML::load(DATA)

  ONELINERS.each do |name, code|
    define_method( name, &eval("lambda { #{code} }") )
  end

  def oneliner(name)
    ONELINERS[name.to_s]
  end

  def oneliners
    ONELINERS.keys
  end

end


if $0 == __FILE__
  RubyOneLiners.color_ruby
end


__END__

color_ruby: |
  "jp6iSZmkLp5ISZlEiW5C".unpack("m")[0].unpack("C*").map{|x|x.chr}.join.
  unpack("B*")[0].scan(/.{24}/){i=7;$&.scan(/..../){print"\e[3#{i-=1};1;40m  ";
  $&.each_byte{|z|print" #"[z-?0,1]*2}};puts"\e[0m"}

quack: |
  s="\e[2J\e[0;0H         _\nQuack! >(*)____,\n        (` =~~/\n^v^v^v^v^`---'v^v^v^v^\n";
  i=0;l{s[21,6]=i&1>0?'Quack!':' '*6;s.tr!(t='*.>v^<,"',t.r);$><<s;i+=s 1}

quack2: |
  s="\e[2J\e[0;0H         _\nQuack! >(*)____,\n        (` =~~/\n^v^v^v^v^`---'v^v^v^v^\n";i=0;
  loop{s[21,6]=i&1>0?'Quack!':' '*6;s.tr!(t='*.>v^<,"',t.reverse);$><<s;i+=sleep 1}

santa: |
  loop{7.times{|i|puts"\e[2J\e[0;3H_...\n / _  '.\n|.'(@@@@@)\n(*)/ %s \\
     | .o. |  \n   (  %s  ) %s"%[i==5?"- -":"e e",(i+3)%7<4?"=":"o",i>0?"-"+
  (" Ho"*i)[0,9]+"!":""];puts"   (     )\n"*2,"    (   )\n"*3,'     "-"';
  sleep([5,3,3,4,5,1,4][i]/6.0)}}

# This one can take two parameter but block aren't nice about defaults :(
fractree: |
  n||=32;l||="A";
  #n.times{|y|print" "*(n-1-y);(y+1).times{|x|print~y&x>0?" .":" #{l[0.1]}"};puts}
  n.times{|y|print" "*(n-1-y),(0..y).map{|x|~y&x>0?" .":" #{l}"},$/}

fractree2: |
  a=[];10.times{|n|a<<1;puts" "*(9-n)*3+"%6d"*(n+1)%a;n.times{|i|a[n]+=a[n-=1]}}

fractree3: |
  a=[];10.times{|n|a<<1;puts" "*(9-n)*3+"%6d"*-~n%a;n.times{|i|a[n]+=a[n-=1]}}

duck: |
  s="\033[2J\033[0;0H          _\n Quack! >(')____,\n         (` =~~/\n~^~^~^~^~^`---'~^~^~^~^";
  (1..(1/0.0)).each{|i|s[23,6]=(i%2)==0?"Quack!":" "*6;s.tr!('>~^^~<','<^~~^>');puts s;sleep 1}

duck2: |
  s="\e[2J\e[0;0H         _\nQuack! >(*)____,\n        (` =~~/\n^v^v^v^v^`---'v^v^v^v^\n";
  i=0;loop{s[21,6]=i&1>0?'Quack!':' '*6;s.tr!(t='*.>v^<,"',t.reverse);$><<s;i+=sleep 1}

duck3: |
  q,w='Quack! >','~^'*11;t="\e[2J\e[0;0H         _\n%8s(')____,\n        (` =^^/\n%20s\n";
  i=0;loop{s=t%[i&1>0?q:'<',w[i%2,20]];$><<s;i+=sleep 1}

duck4: |
  i=1;loop{puts"\e[2J\e[0;0H#{' '*10}_\n %8s(*)____,\n#{' '*9}
  (` =~~/\n#{'~^~'[i%2,2]*9}"%(i&1>0?'<':'Quack! >');i+=sleep 1}

ruby_quiz: |
  "bp6siZmijp5CiZlCiW5CgAAChpbiiZYiiZZCi5aCZ2bs".unpack("m")[0].unpack("C*").map{|x|
  x.chr}.join.unpack("B*")[0].scan(/.{24}/){i=7;$&.scan(/..../){
  print"\e[3#{i-=1};1;40m  ";$&.each_byte{|z|print" #"[z-?0,1]*2}};puts"\e[0m"}
