
# Credit goes to Daniel J. Berger, via PTools.
# The nl_convert() method was adopted (somewhat) from the nlcvt program
# found at http://www.perl.com/language/ppt/src/nlcvt/nlcvt, written by
# Tom Christiansen.

class FileUitls

  # Converts a text file from one OS platform format to another, ala
  # ((*dos2unix*)).  Valid values for ((*format*)), which are case insensitve,
  # includes:
  #
  # * MS Windows -> dos, windows, win32
  # * Unix/BSD   -> unix, linux, bsd
  # * Mac        -> mac, macintosh, apple, osx
  #
  # Note that this method is only valid for an ftype of "file".  Otherwise a
  # TypeError will be raised.  If an invalid format value is received,
  # StandardError is raised.
  #
  #   # Creates a copy of 'myfile' called 'newfile', in DOS format
  #   File.nl_convert("myfile","newfile","dos")
  #
  # (Note: Looking to improve.)

  def self.nl_convert(old_file,new_file=old_file,platform="dos")

    unless File.ftype(old_file) == "file"
        raise TypeError, "Only valid for plain text files"
    end

    if platform =~ /dos|windows|win32/i
        format = "\cM\cJ"
    elsif platform =~ /unix|linux|bsd/i
        format = "\cJ"
    elsif platform =~ /mac|apple|macintosh|osx/i
        format = "\cM"
    else
        raise StandardError, "Invalid platform string"
    end

    orig = $\
    $\ = format

    if old_file == new_file
        require "ftools"
        require "tempfile"
        tf = Tempfile.new("temp")
        tf.open
        IO.foreach(old_file){ |line|
          line.chomp!
          tf.print line
        }
        tf.close
        File.delete(old_file)
        File.copy(tf.path,old_file)
    else
        nf = File.new(new_file,"w+")  
        IO.foreach(old_file){ |line|
          line.chomp!
          nf.print line
        }
        nf.close
    end

    $\ = orig
  end

end



=begin

require 'test/unit'
require 'ftools'

class TC_FileNLConvert < Test::Unit::TestCase

   @@tfile   = 'test_file.txt'
   @@tfile2  = 'test_file2.txt'
   @@dosfile = 'dos_test_file.txt'
   @@macfile = 'mac_test_file.txt'
   @@nixfile = 'nix_test_file.txt'

   def test_01_method_responds
      assert_respond_to( File, :nl_convert )
   end

   def test_02_constructor
      assert_nothing_raised{ File.nl_convert(@@tfile2) }
      assert_nothing_raised{ File.nl_convert(@@tfile2,@@tfile2) }
      assert_nothing_raised{ File.nl_convert(@@tfile2,@@tfile2,"unix") }
      File.copy(@@tfile,@@tfile2)
   end

   def test_03_dos
      assert_nothing_raised{ File.nl_convert(@@tfile,@@dosfile,'dos') }
      err = "dos file should be larger, but isn't"
      assert_equal(true, File.size(@@dosfile) > File.size(@@tfile),err)
   end
   
   def test_04_nix
      assert_nothing_raised{ File.nl_convert(@@tfile,@@nixfile,'unix') }
      err = "unix file should be the same size (or smaller), but isn't"
      if File::ALT_SEPARATOR
         assert_equal(true, File.size(@@nixfile) >= File.size(@@tfile),err)
      else
         assert_equal(true, File.size(@@nixfile) <= File.size(@@tfile),err)
      end
   end
   
   # May fail on MS Windows - no idea why
   def test_05_mac
      assert_nothing_raised{ File.nl_convert(@@tfile,@@macfile,'mac') }
      err = "mac file should be the same size (or larger), but isn't"
      assert_equal(true, File.size(@@macfile) >= File.size(@@tfile),err)
   end

   def test_06_exceptions
      assert_raises(StandardError){ File.nl_convert(@@tfile,"bogus.txt","blah") }
   end

   def teardown
      [@@dosfile,@@macfile,@@nixfile].each{ |f|
         File.delete(f) if File.exists?(f)
      }
   end
end

=end


